#import "FlulmAuthPlugin.h"
#import "LemaAPI.h"
#import "FluwxStringUtil.h"
#import "FluwxAuthHandler.h"
#import <LMPay/LMPay.h>

@implementation FlulmAuthPlugin
FluwxAuthHandler *_fluwxAuthHandler;


BOOL isWeChatRegistered = NO;



+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"com.awesome/flulm_auth"
            binaryMessenger:[registrar messenger]];
    
  FlulmAuthPlugin* instance = [[FlulmAuthPlugin alloc] initWithRegistrar:registrar methodChannel:channel];
  [registrar addApplicationDelegate:instance];
  [registrar addMethodCallDelegate:instance channel:channel];

}

- (instancetype)initWithRegistrar:(NSObject <FlutterPluginRegistrar> *)registrar methodChannel:(FlutterMethodChannel *)flutterMethodChannel {
    
    self = [super init];
    if (self) {
        _fluwxAuthHandler = [[FluwxAuthHandler alloc] initWithRegistrar:registrar methodChannel:flutterMethodChannel];;
    }

    return self;
}




- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  }else if ([@"registerApp" isEqualToString:call.method]) {
      printf("registerApp==>");
      [self registerApp:call result:result];
   } else if ([@"sendAuth" isEqualToString:call.method]) {
        printf("sendAuth==>");
       [_fluwxAuthHandler handleAuth:call result:result];
  }else if ([@"isLemaPayInstalled" isEqualToString:call.method]) {
      printf("call isLemaPayInstalled====");
      BOOL installed = [[LMPay shareInstance] isLemaInstalled];
      result(@(installed));
  }else if ([@"isLemaPaySupportAPI" isEqualToString:call.method]) {
      printf("call isLemaPaySupportAPI===");
      BOOL isSupported = NSClassFromString(@"LMPay") != nil;
      result(@(isSupported));
  } else if ([@"sendPay" isEqualToString:call.method]) {
      NSLog(@"call sendPay==> %@", call.arguments);
      [self lemaPay:call.arguments result: result];
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (NSString *)lemaURLScheme {
    NSArray *types = NSBundle.mainBundle.infoDictionary[@"CFBundleURLTypes"];
    for (NSDictionary *scheme in types) {
        if ([[scheme[@"CFBundleURLName"] lowercaseString] isEqualToString: @"lmpay"]) {
            return [scheme[@"CFBundleURLSchemes"] firstObject] ?: @"";
        }
    }
    return @"";
}

-(BOOL)application:(UIApplication *)application openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options{
    if ([[LMPay shareInstance] handleOpenURL:url]) {
        // 乐马支付处理结果
    } else {
        [_fluwxAuthHandler handelUrl:url];
    }
    return NO;
}

- (void)registerApp:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (!call.arguments[@"iOS"]) {
        result(@NO);
        return;
    }

    NSString *appId = call.arguments[@"appId"];
    if ([FluwxStringUtil isBlank:appId]) {
        result([FlutterError errorWithCode:@"invalid app id" message:@"are you sure your app id is correct ? " details:appId]);
        return;
    }

    NSString *universalLink = call.arguments[@"universalLink"];

//    if ([FluwxStringUtil isBlank:universalLink]) {
//        result([FlutterError errorWithCode:@"invalid universal link" message:@"are you sure your universal link is correct ? " details:universalLink]);
//        return;
//    }

     [LemaAPI registerAPP:appId secret:universalLink];

    result(@YES);
}

- (void)lemaPay:(NSDictionary *)params result:(FlutterResult)result {
    // NSString *openId = call.arguments[@"openId"];
    NSString *signData = params[@"sign_data"];
    NSString *payType = params[@"pay_type"];
    NSString *language = params[@"language"];
    NSInteger isTest = [params[@"isTest"] intValue];
    NSString *urlScheme = [self lemaURLScheme];
    NSInteger netEnvironment = 3;
    switch (isTest) {
        case 0: netEnvironment = 3; break; // 外网生产
        case 1: netEnvironment = 2; break; // 外网测试
        case 2: netEnvironment = 1; break; // 内网测试
    }
    UIViewController *rootVC = UIApplication.sharedApplication.keyWindow.rootViewController;
    [[LMPay shareInstance] initForLemaWithData:signData appType:payType urlSchemes:urlScheme netEnvironment:netEnvironment SDKLanguage:language htmlPushController:rootVC isMustH5:NO errorback:^(NSDictionary * _Nonnull resultDic) {
        NSInteger code = [resultDic[@"code"] integerValue];
        NSInteger payResult = 0;
        switch (code) {
            case 0: // 成功
                payResult = 0;
                break;
            case 200: // LMPay H5加载完成
            case 201:    // LMPay H5开始返回数据
                return;
            case -202: //    退出支付
                if (rootVC.presentedViewController != nil) {
                    [rootVC dismissViewControllerAnimated:YES completion:nil];
                } else {
                    [rootVC.navigationController popViewControllerAnimated:YES];
                }
                payResult = -2;
                break;
            case -100: //    路由名称为空
            case -101: //    LMPay H5加载失败
            case -201: //    未安装乐马Pay
            case -301: //    SDK初始化失败
            case -401: //    初始化接口调用失败
            default:
                payResult = code;
                break;
        }
        result(@{@"platform": @"ios", @"type": payType, @"errCode": @(payResult), @"resultData": resultDic[@"msg"]});
    }];
}

@end
