import 'package:flulm_auth/src/model/result_data.dart';
import 'package:flulm_auth/src/utils/constant.dart';
import 'package:flulm_auth/src/utils/http_manager.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

import 'awesome_web_view.dart';

///**
// * @author dashu
// * @date 6/15/20
// * describe:
// * 授权页面
// */
class AuthPage extends StatefulWidget {
  final String appid;
  final int isTest;
  final String scope;
  final String language;
  final bool supportScan;

  AuthPage(
      {this.scope,
      this.appid,
      this.isTest,
      this.language,
      this.supportScan,
      Key key})
      : super(key: key);

  @override
  _AuthPageState createState() => _AuthPageState();
}

class _AuthPageState extends State<AuthPage> {
  String webUrl = "";
  WebViewController _webViewController;

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<ResultData>(
        future: getLMAuthH5Url(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            if (snapshot.hasError) {
              return SafeArea(
                child: Scaffold(
                  backgroundColor: Theme.of(context).primaryColor,
                  body: Center(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Text(
                          "Error: ${snapshot.data?.msg ?? '获取H5地址失败'}",
                          style: TextStyle(color: Colors.red),
                        ),
                        SizedBox(
                          height: 30.0,
                        ),
                        RaisedButton(
                          onPressed: () {
                            goBackError(
                                snapshot.data.code, snapshot.data.msg, '');
                          },
                          child: Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: Text(
                              '返回',
                              style: TextStyle(
                                fontSize: 30.0,
                                color: Colors.white,
                              ),
                            ),
                          ),
                          color: Theme.of(context).primaryColor,
                        )
                      ],
                    ),
                  ),
                ),
              );
            } else {
              return SafeArea(
                child: Scaffold(
                  body: Container(
                    width: Constant.getResetWidth(context),
                    child: AwesomeWebView(widget.isTest,webUrl,widget.language,widget.supportScan),
                  ),
                ),
              );
            }
          } else {
            return Container(
              child: Center(
                child: CircularProgressIndicator(
                  backgroundColor: Colors.white,
                ),
              ),
              color: Colors.blue,
            );
          }
        });
  }

//  /**
//   * 返回键 让前端处理
//   */
  void _loadBackPressed() {
    _webViewController?.evaluateJavascript('onBackPressed()')?.then((result) {
      // You can handle JS result here.
      debugPrint("_loadBackPressed=" + result);
      if (result == null) {
        goBackError(-1, "用户已取消", "");
      }
    });
  }

  Future<ResultData> getLMAuthH5Url() async {
    String host;
    switch (widget.isTest) {
      case 1:
        host = "https://testoauthapi.lmpayplc.com";
        break;
      case 2:
        host = "http://oauthapi1.lmpayplc.com";
        break;
      case 0:
      default:
        host = "https://oauthapi.lmpayplc.com";
    }

    Map<String, String> jsonMap = Map();
    jsonMap["appid"] = widget.appid;
    jsonMap["scope"] = widget.scope;
    jsonMap["state"] = "state";
    jsonMap["no_redirect_uri"] = "1";
    jsonMap["response_type"] = "code";
    jsonMap["from"] = "flutter-" + defaultTargetPlatform.toString();
    ResultData resultData = await HttpManager.getInstance(host)
        .get("?ct=index&ac=connect", jsonMap);
    webUrl = resultData.data["h5_url"];
    if (webUrl?.isNotEmpty == true) {
      webUrl = webUrl + "&noscan=" + (widget.supportScan ? 0 : 1).toString();
    }
    print("getLMAuthH5Url:" + webUrl);
    return resultData;
  }

  void goBackError(int errCode, String msg, String code) {
    Map<String, dynamic> map = Map();
    map['errCode'] = errCode;
    map['errStr'] = msg;
    //授权的code
    map['code'] = code;
    Navigator.pop(context, map);
  }

  void closePage(int status) {
    debugPrint("closePage == status=" + status.toString());
    if (status == 0) {
      Navigator.pop(context);
    } else {
      showExitDialog();
    }
  }

//  @JavascriptInterface
//  public void getLoginCodeSuccess(String code) {
//    goBack2ThirdApp(0, "授权成功", code);
//  }
//
  void onAuthResultCall(int errorCode, String message, String code) {
    goBackError(errorCode, message, code);
  }

  void showExitDialog() {
    String title = "Are you sure to exit?";
    String btn = "Exit";
    if (widget.language == "zh-cn") {
      title = "是否确定退出？";
      btn = "退出";
    } else if (widget.language == "km") {
      title = "តើអ្នកប្រាកដជាចាកចេញឬ?";
      btn = "បោះបង់ការសិក្សា";
    }
    AlertDialog(
      content: Text(title),
      actions: <Widget>[
//        FlatButton(
//          child: Text("取消"),
//          onPressed: () => Navigator.of(context).pop(), //关闭对话框
//        ),
        FlatButton(
          child: Text(btn),
          onPressed: () {
            Navigator.pop(context); //关闭对话框
          },
        ),
      ],
    );
  }
}
